package gssim.schedframe.scheduling.plugin.grid;

import eduni.simjava.Sim_event;
import gridsim.GridResource;
import gridsim.GridSim;
import gridsim.GridSimCore;
import gridsim.ParameterException;
import gridsim.gssim.network.ExtendedGridSim;
import gridsim.gssim.network.ExtendedRouter;
import gridsim.gssim.network.flow.reservation.GSSIMFlowLink;
import gridsim.gssim.network.flow.reservation.GSSIMFlowRouter;
import gridsim.net.FIFOScheduler;
import gssim.schedframe.scheduling.plugin.grid.network.manager.NetworkReservationObject;
import gssim.schedframe.scheduling.plugin.grid.network.manager.ReservationIntervalCharacteristic;
import gssim.schedframe.scheduling.plugin.grid.network.manager.NetworkReservationManager;
import gssim.schedframe.scheduling.plugin.grid.network.manager.Topology;
import gssim.schedframe.scheduling.plugin.grid.network.pce.PCE4Simulator;
import gssim.schedframe.scheduling.plugin.grid.network.utils.BellmanFord;
import gssim.schedframe.scheduling.plugin.grid.network.utils.BellmanFord.Edge;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import org.joda.time.DateTime;

import schedframe.exceptions.NetworkException;
import schedframe.net.Path;
import schedframe.net.pce.topology.AbstractTopology;
import schedframe.net.pce.topology.AvailableBandwidth;
import schedframe.net.pce.topology.DataLink;
import schedframe.net.pce.topology.ExtendedDataLink;
import schedframe.net.pce.topology.ExtendedVirtualLink;
import schedframe.net.pce.topology.Node;
import schedframe.net.pce.topology.NodeType;
import schedframe.net.pce.topology.VirtualLink;
import schedframe.scheduling.plugin.grid.ModuleType;
import schedframe.scheduling.plugin.grid.NetworkManager;
import simulator.GridBroker;


public class GSSIMNetworkManager implements NetworkManager {

	//protected final int MILLI_SEC = 1000;

	private int userID;
	private NetworkReservationManager reservationManager;
	private AbstractTopology abstractTopology;
	private PCE4Simulator simulator;

	public GSSIMNetworkManager(int userID, String topologyFileName) {
		this.userID = userID;
		this.reservationManager = new NetworkReservationManager(userID,
				topologyFileName);
		this.abstractTopology = new AbstractTopology();
		createAbstractTopology(NetworkReservationManager.getTopology());
		this.simulator = new PCE4Simulator();

	}

	public GSSIMNetworkManager(int userID, Topology topology) {
		this.userID = userID;

		this.reservationManager = new NetworkReservationManager(userID,
				topology);
		this.abstractTopology = new AbstractTopology();
		createAbstractTopology(NetworkReservationManager.getTopology());
		this.simulator = new PCE4Simulator();

	}
	
	public boolean cancelReservation(int reservationID) throws NetworkException{
		boolean status = false;
		try {
			status = reservationManager.cancelReservation(reservationID);
		} catch (NetworkException e) {
			throw e;
		}
		return status;
	}

	public int createReservation(String node1Name, String node2Name,
			DateTime startDate, DateTime finishDate, long bandwidth) throws NetworkException {
		if (node1Name == null || node2Name == null || startDate == null
				|| finishDate == null) {
			throw new NetworkException(new NullPointerException());
		}
		updateExtendedDataLink();
		List<String> links = null;
		try {
			links = simulator.findShortestPathJodaTime(abstractTopology,
					getAbstractTopologyNode(node1Name),
					getAbstractTopologyNode(node2Name), bandwidth, startDate,
					finishDate);
		} catch (Exception e) {
			throw new NetworkException("There are no links which satisfy bandwidth requirements");
		}

		if (links == null || links.size() == 0)
			throw new NetworkException("There are no links which satisfy bandwidth requirements");
		
		int reservationID = -1;
		try{
			reservationID = reservationManager.createReservation(userID, startDate.getMillis(), finishDate.getMillis(),
					bandwidth, links);
		} catch (NetworkException e) {
			throw e;
		}
		return reservationID;
	}

	public int createReservation(String node1Name, String node2Name, Path path,
			DateTime startDate, DateTime finishDate, long bandwidth) throws NetworkException{
		if (node1Name == null || node2Name == null || path == null
				|| startDate == null || finishDate == null) {
			throw new NetworkException(new NullPointerException());
		}
		if (node1Name.compareTo(path.getNode1Name()) != 0
				&& node1Name.compareTo(path.getNode2Name()) != 0)
			throw new NetworkException("Wrong path");
		if (node2Name.compareTo(path.getNode1Name()) != 0
				&& node2Name.compareTo(path.getNode2Name()) != 0)
			throw new NetworkException("Wrong path");
		
		int reservationID = -1;
		try{
			reservationID = reservationManager.createReservation(userID, startDate.getMillis(),
				finishDate.getMillis(), bandwidth, path.getLinks());
		}catch (NetworkException e) {
			throw e;
		}
		return reservationID;
	}

	public long getBandwidth(String node1Name, String node2Name) {
		if (node1Name == null || node2Name == null) {
			return 0;
		}
		updateExtendedDataLink();
		List virtualLinks = null;
		try {
			virtualLinks = simulator.getGlobalMaxFlowMatrix(abstractTopology);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		if (virtualLinks == null)
			return 0;
		long bandwidth = 0;
		Iterator<VirtualLink> it = virtualLinks.iterator();
		while (it.hasNext()) {
			VirtualLink virtualLink = (VirtualLink) it.next();
			{
				List<AvailableBandwidth> calendar = virtualLink.getCalendar();
				Iterator<AvailableBandwidth> it2 = calendar.iterator();
				bandwidth = calendar.get(0).getValue();
			
				while (it2.hasNext()) {
					AvailableBandwidth availableBandwidth = (AvailableBandwidth) it2
							.next();
					//System.out.println(virtualLink.getEndpoint1().getId()+";"+virtualLink.getEndpoint2().getId()+";"+availableBandwidth.getTimestamp()+";"+availableBandwidth.getValue());
				}
			}	
		}
		it = virtualLinks.iterator();
		while (it.hasNext()) {
			VirtualLink virtualLink = (VirtualLink) it.next();

			if ((virtualLink.getEndpoint1().getId().compareTo(node1Name) == 0 && virtualLink
					.getEndpoint2().getId().compareTo(node2Name) == 0)
					|| (virtualLink.getEndpoint2().getId().compareTo(node1Name) == 0 && virtualLink
							.getEndpoint1().getId().compareTo(node2Name) == 0)) {
				List<AvailableBandwidth> calendar = virtualLink.getCalendar();
				Iterator<AvailableBandwidth> it2 = calendar.iterator();
				bandwidth = calendar.get(0).getValue();
				
				while (it2.hasNext()) {
					AvailableBandwidth availableBandwidth = (AvailableBandwidth) it2
							.next();

					if (ExtendedGridSim.getCurrentTime() < availableBandwidth.getTimestamp()) 
						break;

					bandwidth = availableBandwidth.getValue();

				}
				break;
			}
		}
		return bandwidth;
	}

	public long getBandwidth(String node1Name, String node2Name,
			DateTime startDate, DateTime finishDate) {
		if (node1Name == null || node2Name == null || startDate == null
				|| finishDate == null) {
			return 0;
		}
		updateExtendedDataLink();
		List virtualLinks = null;
		try {
			virtualLinks = simulator.getGlobalMaxFlowMatrix(abstractTopology);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		if (virtualLinks == null)
			return 0;
		long bandwidth = 0;
		long minFreeBandwidth = 0;
		long startTime = startDate.getMillis();
		long finishTime = finishDate.getMillis();
		Iterator<VirtualLink> it = virtualLinks.iterator();
		while (it.hasNext()) {
			VirtualLink virtualLink = (VirtualLink) it.next();
			if ((virtualLink.getEndpoint1().getId().compareTo(node1Name) == 0 && virtualLink
					.getEndpoint2().getId().compareTo(node2Name) == 0)
					|| (virtualLink.getEndpoint2().getId().compareTo(node1Name) == 0 && virtualLink
							.getEndpoint1().getId().compareTo(node2Name) == 0)) {
				List<AvailableBandwidth> calendar = virtualLink.getCalendar();
				Iterator<AvailableBandwidth> it2 = calendar.iterator();
				bandwidth = calendar.get(0).getValue();
				minFreeBandwidth = bandwidth;
				while (it2.hasNext()) {
					AvailableBandwidth availableBandwidth = (AvailableBandwidth) it2
							.next();
					if (startTime < availableBandwidth.getTimestamp()) 
					{	
						if (bandwidth < minFreeBandwidth) {
							minFreeBandwidth = bandwidth;
						}
						if (finishTime > availableBandwidth.getTimestamp()) {
							bandwidth = availableBandwidth.getValue();
						}
						if (finishTime <= availableBandwidth.getTimestamp()) {
							break;
						}
						continue;
					}
					bandwidth = availableBandwidth.getValue();

				}
				break;
			}
		}
		bandwidth = minFreeBandwidth;
		return bandwidth;
	}

	public long getBandwidth(Path path) {
		if (path == null) {
			return 0;
		}
		updateExtendedDataLink();
		long bandwidth = 0;
		long minFreeBandwidth = 0;
		List<DataLink> dataLinks = getAbstractTopologyLinks(path
				.getLinks());
		if (dataLinks == null || dataLinks.size() == 0)
			return 0;
		Iterator<DataLink> it = dataLinks.iterator();
		List<AvailableBandwidth> calendar;
		calendar = dataLinks.get(0).getCalendar();
		if (calendar == null || calendar.size() == 0)
			return 0;
		// minFreeBandwidth = dataLinks.get(0).getBandwidth();
		minFreeBandwidth = calendar.get(0).getValue();
		/*
		 * while (it2.hasNext()) { availableBandwidth = (AvailableBandwidth)
		 * it2.next(); if (getCurrentTime() < availableBandwidth.getTimestamp())
		 * { minFreeBandwidth = availableBandwidth.getValue(); break; } }
		 */
		while (it.hasNext()) {
			DataLink dataLink = (DataLink) it.next();
			calendar = dataLink.getCalendar();
			Iterator<AvailableBandwidth> it2 = calendar.iterator();
			if (calendar == null || calendar.size() == 0)
				continue;
			while (it2.hasNext()) {
				AvailableBandwidth availableBandwidth = (AvailableBandwidth) it2
						.next();

				if (ExtendedGridSim.getCurrentTime() < availableBandwidth.getTimestamp()) {
					break;
				}

				bandwidth = availableBandwidth.getValue();
				if (bandwidth < minFreeBandwidth) {
					minFreeBandwidth = bandwidth;
				}
	
			}
		}
		bandwidth = minFreeBandwidth;
		return bandwidth;
	}

	public long getBandwidth(Path path, DateTime startDate, DateTime finishDate) {
		if (path == null || startDate == null || finishDate == null) {
			return 0;
		}
		updateExtendedDataLink();
		long bandwidth = 0;
		long minFreeBandwidth = 0;
		long startTime = startDate.getMillis();
		long finishTime = finishDate.getMillis();
		List<DataLink> dataLinks = getAbstractTopologyLinks(path
				.getLinks());
		if (dataLinks == null || dataLinks.size() == 0)
			return 0;
		Iterator<DataLink> it = dataLinks.iterator();
		List<AvailableBandwidth> calendar;
		calendar = dataLinks.get(0).getCalendar();
		if (calendar == null || calendar.size() == 0)
			return 0;
		
		// minFreeBandwidth = dataLinks.get(0).getBandwidth();
		minFreeBandwidth = calendar.get(0).getValue();

		while (it.hasNext()) {
			DataLink dataLink = (DataLink) it.next();
			//System.out.println(dataLink.getId());
			calendar = dataLink.getCalendar();
			if (calendar == null || calendar.size() == 0)
				continue;
			Iterator<AvailableBandwidth> it2 = calendar.iterator();
			// minFreeBandwidth = calendar.get(0).getValue();

			while (it2.hasNext()) {
				AvailableBandwidth availableBandwidth = (AvailableBandwidth) it2
				.next();
				if (startTime < availableBandwidth.getTimestamp()) 
				{	
					if (bandwidth < minFreeBandwidth) {
							minFreeBandwidth = bandwidth;
					}
					if (finishTime > availableBandwidth.getTimestamp()) {
						bandwidth = availableBandwidth.getValue();
					}
					if (finishTime <= availableBandwidth.getTimestamp()) {
						break;
					}
					continue;
				}
				bandwidth = availableBandwidth.getValue();
			}
			// break;
		}
		bandwidth = minFreeBandwidth;
		return bandwidth;
	}

	public List<ExtendedVirtualLink> getBandwidthMatrix() {
		List<ExtendedVirtualLink> extendedVirtualLinks = null;
		updateExtendedDataLink();
		try {
			extendedVirtualLinks = simulator
					.getGlobalMaxFlowMatrix(abstractTopology);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}

		Iterator<ExtendedVirtualLink> it = extendedVirtualLinks.iterator();
		ExtendedVirtualLink extendedVirtualLink;
		while (it.hasNext()) {
			extendedVirtualLink = (ExtendedVirtualLink) it.next();

			List<AvailableBandwidth> calendar = extendedVirtualLink
					.getCalendar();
			Iterator<AvailableBandwidth> it2 = calendar.iterator();
			AvailableBandwidth availableBandwidth;
			while (it2.hasNext()) {
				
				
				availableBandwidth = (AvailableBandwidth) it2.next();
				if (ExtendedGridSim.getCurrentTime() < availableBandwidth.getTimestamp()) {
					break;
				}
				extendedVirtualLink.setCurrentBandwidth(availableBandwidth
						.getValue());

			}
			extendedVirtualLink
					.setCalendar(new ArrayList<AvailableBandwidth>());
			// extendedVirtualLinks.add(extendedVirtualLink);
		}
		return extendedVirtualLinks;
	}

	public List<ExtendedVirtualLink> getBandwidthMatrix(DateTime startDate,
			DateTime finishDate) {
		if (startDate == null || finishDate == null) {
			return null;
		}
		updateExtendedDataLink();
		List<ExtendedVirtualLink> extendedVirtualLinks = null;
		try {
			extendedVirtualLinks = simulator
					.getGlobalMaxFlowMatrix(abstractTopology);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		long bandwidth = 0;
		long minFreeBandwidth = 0;
		long startTime = startDate.getMillis();
		long finishTime = finishDate.getMillis();
		Iterator<ExtendedVirtualLink> it = extendedVirtualLinks.iterator();
		ExtendedVirtualLink extendedVirtualLink;
		while (it.hasNext()) {
			extendedVirtualLink = (ExtendedVirtualLink) it.next();

			List<AvailableBandwidth> calendar = extendedVirtualLink
					.getCalendar();
			Iterator<AvailableBandwidth> it2 = calendar.iterator();
			AvailableBandwidth availableBandwidth;
			if (calendar != null && calendar.size() != 0)
				minFreeBandwidth = calendar.get(0).getValue();

			while (it2.hasNext()) {
				availableBandwidth = (AvailableBandwidth) it2
				.next();
				if (startTime < availableBandwidth.getTimestamp()) 
				{	
					if (bandwidth < minFreeBandwidth) {
						minFreeBandwidth = bandwidth;
					}
					if (finishTime > availableBandwidth.getTimestamp()) {
						bandwidth = availableBandwidth.getValue();
					}
					if (finishTime <= availableBandwidth.getTimestamp()) {
						break;
					}
					continue;
				}
				bandwidth = availableBandwidth.getValue();
			}
			extendedVirtualLink.setCurrentBandwidth(minFreeBandwidth);
			extendedVirtualLink
					.setCalendar(new ArrayList<AvailableBandwidth>());
			// extendedVirtualLinks.add(extendedVirtualLink);

		}
		return extendedVirtualLinks;
	}

	public List<ExtendedVirtualLink> getAbsolutBandwidthMatrix() {
		List<ExtendedVirtualLink> extendedVirtualLinks = null;
		updateExtendedDataLink();
		try {
			extendedVirtualLinks = simulator
					.getGlobalMaxFlowMatrix(abstractTopology);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		Iterator<ExtendedVirtualLink> it = extendedVirtualLinks.iterator();
		ExtendedVirtualLink extendedVirtualLink;
		while (it.hasNext()) {
			extendedVirtualLink = (ExtendedVirtualLink) it.next();
			extendedVirtualLink.setCurrentBandwidth(-1);
		}
		return extendedVirtualLinks;
	}

	public double getLatency(String node1Name, String node2Name) {
		if (node1Name == null || node2Name == null) {
			return 0;
		}
		Vector<Edge> edges = new Vector<Edge>();
		HashMap<String, Integer>nodeMap = new HashMap<String, Integer>();
		List linksList = abstractTopology.getLinks();
		Iterator it = linksList.iterator();
		int value = 0;
		while(it.hasNext()){
			ExtendedDataLink edl = (ExtendedDataLink)it.next();
			if(!nodeMap.containsKey(edl.getEndpoint1().getId())){
				nodeMap.put(edl.getEndpoint1().getId(), value);
				value++;
			}
			if(!nodeMap.containsKey(edl.getEndpoint2().getId())){
				nodeMap.put(edl.getEndpoint2().getId(), value);
				value++;
			}
			edges.add(new Edge((Integer)nodeMap.get(edl.getEndpoint1().getId()), (Integer)nodeMap.get(edl.getEndpoint2().getId()), (int)edl.getLatency()));
			edges.add(new Edge((Integer)nodeMap.get(edl.getEndpoint2().getId()), (Integer)nodeMap.get(edl.getEndpoint1().getId()), (int)edl.getLatency()));
		}
		int []distance = BellmanFord.bellmanFord(edges, abstractTopology.getNodes().size(), (Integer)nodeMap.get(node1Name));
		if(distance.length <= 1)
			return 0;
		double latency = 0;
		for(int i = 0; i < distance.length; i++){
			if((Integer)nodeMap.get(node2Name) == i)
			{
				latency = distance[i];
				break;
			}
		}
		//System.out.println(nodeMap);
		return latency;
	}

	public double getLatency(Path path) {
		if (path == null) {
			return 0;
		}

		double latency = 0;
		List<DataLink> dataLinks = getAbstractTopologyLinks(path
				.getLinks());
		if (dataLinks == null || dataLinks.size() == 0)
			return 0;
		Iterator<DataLink> it = dataLinks.iterator();
		while (it.hasNext()) {
			ExtendedDataLink link = (ExtendedDataLink) it.next();
			latency += link.getLatency();
		}
		return latency;
	}

	public double[][] getLatencyMatrix() {
		// TODO Auto-generated method stub
		return null;
	}

	/*public Topology getTopology() {
		return NetworkReservationManager.getTopology();
	}*/

	public boolean modifyReservation(int reservationID, DateTime startDate,
			DateTime finishDate, long bandwidth) throws NetworkException{
		boolean status = false;
		try {
			status = reservationManager.modifyReservation(reservationID, startDate.getMillis(),
					finishDate.getMillis(), bandwidth);
		} catch (NetworkException e) {
			throw e;
		}
		return status;
	}

	public List<String> getReservedLinksNames(int reservationID){
		return reservationManager.getReservedLinksNames(reservationID);
	}
	
	public double getReservationStartTime(int reservationID){
		return reservationManager.getReservationStartTime(reservationID);
	}
	
	public double getReservationEndTime(int reservationID){
		return reservationManager.getReservationEndTime(reservationID);
	}
	
	public double getReservationBandwidth(int reservationID){
		return reservationManager.getReservationBandwidth(reservationID);
	}
	
	public List<NetworkReservationObject> getReservations(){
		return reservationManager.getCreatedReservations();
	}

	public AbstractTopology getAbstractTopology() {
		return this.abstractTopology;
	}

	private Node getAbstractTopologyNode(String name) {
		List<Node> abstractNodes = abstractTopology.getNodes();

		Iterator<Node> it = abstractNodes.iterator();
		while (it.hasNext()) {
			Node node = (Node) it.next();

			if (node.getId().compareTo(name) == 0)
				return node;
		}
		return null;

	}

	private ExtendedDataLink getAbstractTopologyLink(String linkName) {
		List<DataLink> abstractLinks = abstractTopology.getLinks();
		Iterator<DataLink> it = abstractLinks.iterator();
		while (it.hasNext()) {
			ExtendedDataLink link = (ExtendedDataLink) it.next();
			if (link.getId().compareTo(linkName) == 0)
				return link;
		}
		return null;
	}

	private List<DataLink> getAbstractTopologyLinks(List<String> linkNames) {
		List<DataLink> abstractLinks = new ArrayList();
		Iterator<DataLink> it = abstractTopology.getLinks().iterator();
		while (it.hasNext()) {
			DataLink link = (DataLink) it.next();
			if (linkNames.contains(link.getId()))
				abstractLinks.add(link);
		}
		return abstractLinks;
	}

	public Path createPath(List<String> linksNames) {
		if (linksNames == null || linksNames.size() == 0)
			return null;
		List abstractTopologyLinks = getAbstractTopologyLinks(linksNames);
		if (abstractTopologyLinks.size() != linksNames.size()) {
			return null;
		}
		ExtendedDataLink link;
		String node1Name;
		String node2Name;
		List<String> pathLinks = new LinkedList<String>();
		link = (ExtendedDataLink) abstractTopologyLinks.remove(0);
		pathLinks.add(link.getId());
		node1Name = link.getEndpoint1().getId();
		node2Name = link.getEndpoint2().getId();
		int counter = abstractTopologyLinks.size();
		boolean matching = false;
		for (int i = 0; i < counter; i++) {
			Iterator<DataLink> it = abstractTopologyLinks.iterator();
			while (it.hasNext()) {
				link = (ExtendedDataLink) it.next();
				matching = false;
				if (link.getEndpoint1().getId().compareTo(node1Name) == 0) {
					node1Name = link.getEndpoint2().getId();
					matching = true;
				} else if (link.getEndpoint2().getId().compareTo(node1Name) == 0) {
					node1Name = link.getEndpoint1().getId();
					matching = true;
				}
				if (link.getEndpoint1().getId().compareTo(node2Name) == 0) {
					node2Name = link.getEndpoint2().getId();
					matching = true;
				} else if (link.getEndpoint2().getId().compareTo(node2Name) == 0) {
					node2Name = link.getEndpoint1().getId();
					matching = true;
				}
				if (matching) {
					abstractTopologyLinks.remove(link);
					pathLinks.add(link.getId());
					break;
				}
			}
			if (!matching)
				return null;

		}
		return new Path(node1Name, node2Name, pathLinks);

	}
	
	/*private List<DataLink> allowForReversedNamesOfLinks(List<String> linksNames){

		List<DataLink> abstractLinks = new ArrayList<DataLink>();
		
		for(DataLink dataLink:abstractTopology.getLinks()){
			for(String linkName:linksNames){
				if(linkName.contains(dataLink.getId())){
					abstractLinks.add(dataLink);
					break;
				}
				else if(linkName.contains(dataLink.getEndpoint1().getId()) && linkName.contains(dataLink.getEndpoint2().getId())){
					abstractLinks.add(dataLink);
					break;
				}
			}
		}
		return abstractLinks;
	}*/
	
	private void updateExtendedDataLink() {

		ExtendedDataLink extendedDataLink = null;

		Iterator it = abstractTopology.getLinks().iterator();

		while (it.hasNext()) {

			extendedDataLink = (ExtendedDataLink) it.next();

			List<AvailableBandwidth> calendar = ((GSSIMFlowLink) (NetworkReservationManager
					.getTopology().getTopologyLink(extendedDataLink.getId())))
					.getBandwidthCalendar();
			if (calendar.size() != 0)
			{
				/*calendar.add(new AvailableBandwidth(14000,
						(long) ((GSSIMFlowLink) (reservationManager.getTopology()
								.getTopologyLink(extendedDataLink.getId())))
								.getStartBaduRate()));*/
				extendedDataLink.setCalendar(calendar);
				Iterator it2 = calendar.iterator();
				while(it2.hasNext()){
					AvailableBandwidth ab = (AvailableBandwidth)it2.next();
					//System.out.println(extendedDataLink.getId()+" UPDATE "+ab.getTimestamp()+";"+ab.getValue());
				}
				Iterator it3 = extendedDataLink.getCalendar().iterator();
				while(it3.hasNext()){
					AvailableBandwidth ab = (AvailableBandwidth)it3.next();
					//System.out.println(extendedDataLink.getId()+"AFTER UPDATE "+ab.getTimestamp()+";"+ab.getValue());
				}
			}
			else {
				calendar.add(new AvailableBandwidth(0,
						(long) ((GSSIMFlowLink) (reservationManager.getTopology()
								.getTopologyLink(extendedDataLink.getId())))
								.getStartBaduRate()));
				//extendedDataLink.setCalendar(calendar);
				extendedDataLink.setCalendar(calendar);
				/*Iterator it2 = calendar.iterator();
				while(it2.hasNext()){
					AvailableBandwidth ab=(AvailableBandwidth)it2.next();
					System.out.println(extendedDataLink.getId()+"U "+ab.getTimestamp()+";"+ab.getValue());
				}*/
			}

		}
	}

	private void createAbstractTopology(Topology topology)
	{
		//abstractTopology = new AbstractTopology();

		Iterator it = topology.getUsers().iterator();
		while (it.hasNext()) {
			GridSimCore user = (GridSimCore) it.next();
			Node node = new Node(user.get_name(), NodeType.GRID_SITE);
			if (!abstractTopology.getNodes().contains(node))
				abstractTopology.addNode(node);
		}

		it = topology.getUserNames().iterator();
		while (it.hasNext()) {
			String userName = (String) it.next();
			Node node = new Node(userName, NodeType.GRID_SITE);
			if (!abstractTopology.getNodes().contains(node))
				abstractTopology.addNode(node);
		}
		
		it = topology.getResourceNames().iterator();
		while (it.hasNext()) {
			String resourceName = (String) it.next();
			Node node = new Node(resourceName, NodeType.GRID_SITE);
			if (!abstractTopology.getNodes().contains(node))
				abstractTopology.addNode(node);
		}
		
		it = topology.getRouters().iterator();
		while (it.hasNext()) {
			GSSIMFlowRouter router = (GSSIMFlowRouter) it.next();
			Node node = new Node(router.get_name(), NodeType.NETWORK_NODE);
			if (!abstractTopology.getNodes().contains(node))
				abstractTopology.addNode(node);
		}
		it = topology.getLinks().iterator();
		Node node1;
		Node node2;
		while (it.hasNext()) {
			GSSIMFlowLink link = (GSSIMFlowLink) it.next();

			if (checkRouters(topology, link.getPoint1Name())) {
				node1 = getAbstractTopologyNode(link.getPoint1Name());
			} else {
				node1 = getAbstractTopologyNode(link.getPoint1Name());
			}
			if (checkRouters(topology, link.getPoint2Name())) {
				node2 = getAbstractTopologyNode(link.getPoint2Name());
			} else {
				node2 = getAbstractTopologyNode(link.getPoint2Name());
			}
			ExtendedDataLink extendedDataLink = new ExtendedDataLink(link
					.get_name(), node1, node2, (long) link.getBaudRate(), link
					.getDelay());
			if (!abstractTopology.getLinks().contains(extendedDataLink))
				abstractTopology.addLink(extendedDataLink);
		}

	}

	private boolean checkRouters(Topology topology, String name) {
		Iterator<ExtendedRouter> it = topology.getRouters().iterator();
		while (it.hasNext()) {
			GSSIMFlowRouter router = (GSSIMFlowRouter) it.next();
			if (router.get_name().compareTo(name) == 0)
				return true;
		}
		return false;
	}

	public void connectUserToRouter(GridSim user, String routerName) {
		FIFOScheduler userSched = null;
		try {
			userSched = new FIFOScheduler("UserSched_" + user.get_id());
		} catch (ParameterException e) {
			e.printStackTrace();
		}
		GSSIMFlowRouter router = (GSSIMFlowRouter) NetworkReservationManager.getTopology()
				.getTopologyRouter(routerName);
		router.attachHost(user, userSched);
		NetworkReservationManager.getTopology().addUser(user);
		GSSIMFlowLink topologyLink = (GSSIMFlowLink) user.getLink();
		topologyLink.setPoints(user, router);
		topologyLink.setPointsNames(user.get_name(), router.get_name());
		NetworkReservationManager.getTopology().addLink(topologyLink);
		Node node = new Node(user.get_name(), NodeType.GRID_SITE);
		if (!abstractTopology.getNodes().contains(node))
			abstractTopology.addNode(node);
		Node node1;
		Node node2;

		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint1().get_name())) {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		} else {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		}
		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint2().get_name())) {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		} else {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		}
		ExtendedDataLink extendedDataLink = new ExtendedDataLink(
				topologyLink.get_name(), node1, node2, (long) topologyLink
						.getBaudRate(), topologyLink.getDelay());
		if (!abstractTopology.getLinks().contains(extendedDataLink))
			abstractTopology.addLink(extendedDataLink);

	}

	public void connectResourceToRouter(GridResource resource, String routerName) {
		FIFOScheduler resourceSched = null;
		try {
			resourceSched = new FIFOScheduler("ResourcerSched_"
					+ resource.get_id());
		} catch (ParameterException e) {
			e.printStackTrace();
		}
		GSSIMFlowRouter router = (GSSIMFlowRouter) NetworkReservationManager.getTopology()
				.getTopologyRouter(routerName);
		router.attachHost(resource, resourceSched);
		NetworkReservationManager.getTopology().addResource(resource);
		GSSIMFlowLink topologyLink = (GSSIMFlowLink) resource.getLink();
		topologyLink.setPoints(resource, router);
		NetworkReservationManager.getTopology().addLink(topologyLink);
		Node node = new Node(resource.get_name(), NodeType.GRID_SITE);
		if (!abstractTopology.getNodes().contains(node))
			abstractTopology.addNode(node);
		Node node1;
		Node node2;

		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint1().get_name())) {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		} else {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		}
		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint2().get_name())) {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		} else {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		}
		ExtendedDataLink extendedDataLink = new ExtendedDataLink(
				topologyLink.get_name(), node1, node2, (long) topologyLink
						.getBaudRate(), topologyLink.getDelay());
		if (!abstractTopology.getLinks().contains(extendedDataLink))
			abstractTopology.addLink(extendedDataLink);

	}

	public void connectToRouter(GridSimCore entity, String routerName) {
		FIFOScheduler resourceSched = null;
		try {
			resourceSched = new FIFOScheduler("ResourcerSched_"
					+ entity.get_id());
		} catch (ParameterException e) {
			e.printStackTrace();
		}
		GSSIMFlowRouter router = (GSSIMFlowRouter) NetworkReservationManager.getTopology()
				.getTopologyRouter(routerName);
		router.attachHost(entity, resourceSched);
		NetworkReservationManager.getTopology().addUser(entity);
		GSSIMFlowLink topologyLink = (GSSIMFlowLink) entity.getLink();
		topologyLink.setPoints(entity, router);
		NetworkReservationManager.getTopology().addLink(topologyLink);
		Node node = new Node(entity.get_name(), NodeType.GRID_SITE);
		if (!abstractTopology.getNodes().contains(node))
			abstractTopology.addNode(node);
		Node node1;
		Node node2;

		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint1().get_name())) {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		} else {
			node1 = getAbstractTopologyNode(topologyLink.getPoint1()
					.get_name());
		}
		if (checkRouters(NetworkReservationManager.getTopology(), topologyLink
				.getPoint2().get_name())) {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		} else {
			node2 = getAbstractTopologyNode(topologyLink.getPoint2()
					.get_name());
		}
		ExtendedDataLink extendedDataLink = new ExtendedDataLink(
				topologyLink.get_name(), node1, node2, (long) topologyLink
						.getBaudRate(), topologyLink.getDelay());
		if (!abstractTopology.getLinks().contains(extendedDataLink))
			abstractTopology.addLink(extendedDataLink);

	}

	public List<String> findShortestPath(String node1Name, String node2Name,
			DateTime startDate, DateTime finishDate, long bandwidth) {
		if (node1Name == null || node2Name == null || startDate == null
				|| finishDate == null) {
			return null;
		}
		updateExtendedDataLink();
		List<String> links = null;
		try {
			links = simulator.findShortestPathJodaTime(abstractTopology,
					getAbstractTopologyNode(node1Name),
					getAbstractTopologyNode(node2Name), bandwidth, startDate,
					finishDate);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return links;
	}

	public List<ReservationIntervalCharacteristic> getNumberOfReservations(String linkName, DateTime timeBorder1, DateTime timeBorder2){
		return reservationManager.getNumberOfReservations(linkName, timeBorder1, timeBorder2);	
	}
	/*private void updateAbstractTopology() {
		createAbstractTopology(reservationManager.getTopology());
	}

	public static HashMap getResLinkMapping(){
		return GSSIMNetworkReader.getResLinkMapping();
	}
	
	public static void  createResourceConnections(){
		GSSIMNetworkReader.createResourceConnections();

	}*/

	public void dispose() {
	}

	public ModuleType getType() {
		return ModuleType.NETWORK_MANAGER;
	}

	public void init(Properties properties) {
	}




}
